package org.xdv.clx.base;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xdv.clx.builder.ClxBuilderException;
import org.xdv.clx.builder.ClxDOMBuilder;
import org.xdv.clx.builder.ClxDOMHelper;
import org.xdv.clx.builder.ClxDOMHelperNS;
import org.xdv.clx.expr.ClxMacroDefinitions;
import org.xdv.clx.expr.ClxRules;
import org.xdv.clx.extension.ClxExtMacroCallPostPrep;
import org.xdv.clx.operator.ClxOperatorCallPostPrep;
import org.xdv.common.UtilDOM;
import org.xdv.xpath.XPathEngine;
import org.xml.sax.SAXException;

/**
 * CLX rules and macros loader.
 * <p>
 * Provides functionality to load rules definitions and macros from different kind of sources:
 * URL, file, stream, W3C DOM element.
 * </p>
 * <p>
 * This implementation wraps {@link ClxDOMBuilder} with all consequences.
 * For instance it means that: it produces {@link ClxExtMacroCallPostPrep} as a result of 
 * macroCall and {@link ClxOperatorCallPostPrep} as a result of operator call. 
 * One should <i>post prepare</i> all those calls before executing any formulas created by 
 * this loader.
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public class ClxLoader implements ClxMacroLoader {
        
    /** XPath engine to create XPath expressions required by some CLX elements */
    protected final XPathEngine xpathEngine;
    
    /** Accept only proper namespace - do not accept elements from <i>empty</i> namespace */
    protected boolean strictNS = false;
    
    /** Skip rule header - rule header will not be loaded, but simply set to <code>null</code>.
     * Default value is <code>false</code>. */
    protected boolean skipHeader = false;
    
    /**
     * Create CLX loader.
     * @param xpathEngine XPath engine to create XPath expressions required by some CLX elements.
     *        <br><b>NOTE:</b> XPath determines type of XML source accepted by formulas during 
     *        execution. 
     */
    public ClxLoader(XPathEngine xpathEngine) {
        this.xpathEngine = xpathEngine;
    }
    
    /**
     * Create CLX loader.
     * @param xpathEngine XPath engine to create XPath expressions required by some CLX elements.
     *        <br><b>NOTE:</b> XPath determines type of XML source accepted by formulas during 
     *        execution.
     * @param strinctNS accept only proper namespace - do not accept elements 
     *        from <i>empty</i> namespace
     */
    public ClxLoader(XPathEngine xpathEngine, boolean strinctNS) {
        this.xpathEngine = xpathEngine;
        this.strictNS = strinctNS;
    }   
    
    /** 
     * Sets skip rule header flag - <code>true</code> rule header will not be loaded, 
     * but simply set to <code>null</code>.
     * @param skipHeader value <code>true</code> means to skip rule header
     */
    public void setSkipHeader(boolean skipHeader) {
        this.skipHeader = skipHeader;
    }
    
    /** 
     * Returns <code>true</code> if rule header is skipped by this loader 
     * If flag is <code>true</code> rule header will not be loaded, 
     * but simply set to <code>null</code>.
     * @return Value <code>true</code> means that rule header is skipped by this loader
     */
    public boolean isSkipHeader() {
        return this.skipHeader;
    }
    
    /**
     * Creates proper {@link ClxDOMBuilder} for given element.
     * <br>
     * Algorithm to create builder is simple:
     * <ol>
     * <li>If <code>root</code> node name is different than <code>rootElemName</code> then
     *     throw {@link ClxBuilderException}.
     * <li>If <code>root</code> comes from {@link ClxConstants#NS_CLIXML} namespace then
     *     return builder with helper for this namespace ({@link ClxDOMHelperNS}.
     * <li>If <code>root</code> comes from <i>empty</i> namespace and {@link #strictNS} is false
     *     then return builder for <i>empty</i> namespace ({@link ClxDOMHelper}).
     * <li>In other case throw {@link ClxBuilderException} 
     * </ol>
     * 
     * @param root CLX root element
     * @param rootElemName expected name for CLX root element, if <code>root</code> has different
     *        name then an {@link ClxBuilderException} will be thrown
     * @return builder CLX builder; !=null
     * @throws ClxBuilderException unable to create builder for given element
     */
    protected ClxDOMBuilder createBuilder(Element root, String rootElemName) 
    throws ClxBuilderException {
        final String rootNS = root.getNamespaceURI();
        ClxDOMBuilder builder;
       
        if ((!this.strictNS) && UtilDOM.isEmptyNS(rootNS) && 
            rootElemName.equals(root.getNodeName())) {
            builder = new ClxDOMBuilder(xpathEngine, ClxDOMHelper.newInstance());
        } else if (ClxConstants.NS_CLIXML.equals(rootNS) &&
                   rootElemName.equals(root.getLocalName())) {
            builder = new ClxDOMBuilder(xpathEngine, ClxDOMHelperNS.newInstance(rootNS));
        } else {
            if (! this.strictNS) {
                throw new ClxBuilderException("Root element must be " + rootElemName, 
                        root.getNodeName());
            }
            throw new ClxBuilderException("Root element must be " + rootElemName + 
                    " in namespace " + ClxConstants.NS_CLIXML, root.getNodeName());
        }
        
        builder.setAllowMacroNoNS(! this.strictNS);
        builder.setAllowReportNoNS(! this.strictNS);
        builder.setSkipHeader(this.skipHeader);
        return builder;
    }
    
    /**
     * Loads rules definitions from element.
     * @param rulesElem rules element. If it has different local-name than 
     *        {@link ClxConstants#TAG_RULES} then {@link ClxBuilderException} is thrown
     * @return rules; !=null
     * @throws ClxBuilderException unable to create rules
     */
    public ClxRules loadRules(Element rulesElem) 
    throws ClxBuilderException {
        ClxDOMBuilder builder = createBuilder(rulesElem, ClxConstants.TAG_RULES);
        return builder.createRules(rulesElem);
    }
    
    /**
     * Loads rules definitions from document.
     * @param clxDoc CLX rule definition document. If root element has different local-name than 
     *        {@link ClxConstants#TAG_RULES} then {@link ClxBuilderException} is thrown
     * @return rules; !=null
     * @throws ClxBuilderException unable to create rules
     */
    public ClxRules loadRules(Document clxDoc) 
    throws ClxBuilderException {
        return this.loadRules(clxDoc.getDocumentElement());
    }
    
    /**
     * Loads rules definitions from file.
     * @param file file name
     * @return rules; !=null
     * @throws ClxBuilderException unable to create rules
     * @throws IOException error while loading data from file 
     */
    public ClxRules loadRules(File file) 
    throws ClxBuilderException, IOException {
        Document doc;
        try {
            doc = UtilDOM.loadXMLusingJAXP(file);
        } catch(ParserConfigurationException e) {
            throw new ClxBuilderException("JAXP XML parser is not properly configured", e);
        } catch(SAXException e) {
            throw new ClxBuilderException("Not valid XML document", e);
        }
        return this.loadRules(doc);
    }
    
    /**
     * Loads rules definitions from stream.
     * @param is stream ready to serve data 
     * @return rules; !=null
     * @throws ClxBuilderException unable to create rules
     * @throws IOException error while loading data from stream
     */
    public ClxRules loadRules(InputStream is) 
    throws ClxBuilderException, IOException {
        Document doc;
        try {
            doc = UtilDOM.loadXMLusingJAXP(is);
        } catch(ParserConfigurationException e) {
            throw new ClxBuilderException("JAXP XML parser is not properly configured", e);
        } catch(SAXException e) {
            throw new ClxBuilderException("Not valid XML document", e);
        }
        return this.loadRules(doc);
    }
    
    /**
     * Loads rules definitions from URL.
     * @param url URL
     * @return rules; !=null
     * @throws ClxBuilderException unable to create rules
     * @throws IOException error while loading data from URL
     */
    public ClxRules loadRules(String url) 
    throws ClxBuilderException, IOException {
        InputStream is = null;
        ClxRules rules;
        try {
            is = new URL(url).openStream();
            rules = this.loadRules(is);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch(Exception e) {
                    //TODO use logger instead of System.err
                    System.err.println("Unable to close stream");
                }
            }
        }
        return rules;
    }
    
    /**
     * Loads macro definitions from element.
     * @param macroDefElem macro definitions element. If it has different local-name than 
     *        {@link ClxConstants#TAG_MACRODEFINITIONS} then {@link ClxBuilderException} is thrown
     * @return macros; !=null
     * @throws ClxBuilderException unable to create macros
     */
    public ClxMacroDefinitions loadMacroDefinitions(Element macroDefElem) 
    throws ClxBuilderException {
        ClxDOMBuilder builder = createBuilder(macroDefElem, ClxConstants.TAG_MACRODEFINITIONS);        
        return builder.createMacroDefinitions(macroDefElem);
    }
    
    /**
     * Loads macro definitions from document.
     * @param clxDoc CLX macro definition document. If root element has different local-name than 
     *        {@link ClxConstants#TAG_MACRODEFINITIONS} then {@link ClxBuilderException} is thrown
     * @return macros; !=null
     * @throws ClxBuilderException unable to create macros
     */
    public ClxMacroDefinitions loadMacroDefinitions(Document clxDoc) 
    throws ClxBuilderException {
        return this.loadMacroDefinitions(clxDoc.getDocumentElement());
    }
    
    /**
     * Loads macro definitions from file.
     * @param file file name
     * @return macros; !=null
     * @throws ClxBuilderException unable to create macros
     * @throws IOException error while loading data from file 
     */
    public ClxMacroDefinitions loadMacroDefinitions(File file) 
    throws ClxBuilderException, IOException {
        Document doc;
        try {
            doc = UtilDOM.loadXMLusingJAXP(file);
        } catch(ParserConfigurationException e) {
            throw new ClxBuilderException("JAXP XML parser is not properly configured", e);
        } catch(SAXException e) {
            throw new ClxBuilderException("Not valid XML document", e);
        }
        return this.loadMacroDefinitions(doc);
    }
    
    /**
     * Loads macro definitions from stream.
     * @param is stream ready to serve data 
     * @return macros; !=null
     * @throws ClxBuilderException unable to create macros
     * @throws IOException error while loading data from stream
     */
    public ClxMacroDefinitions loadMacroDefinitions(InputStream is) 
    throws ClxBuilderException, IOException {
        Document doc;
        try {
            doc = UtilDOM.loadXMLusingJAXP(is);
        } catch(ParserConfigurationException e) {
            throw new ClxBuilderException("JAXP XML parser is not properly configured", e);
        } catch(SAXException e) {
            throw new ClxBuilderException("Not valid XML document", e);
        }
        return this.loadMacroDefinitions(doc);
    }
        
    public ClxMacroDefinitions loadMacroDefinitions(String url) 
    throws ClxBuilderException, IOException {
        InputStream is = null;
        ClxMacroDefinitions macros;
        try {
            is = new URL(url).openStream();
            macros = this.loadMacroDefinitions(is);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch(Exception e) {
                    //TODO use logger instead of System.err
                    System.err.println("Unable to close stream");
                }
            }
        }
        return macros;
    }

}
