package org.xdv.clx.builder;

/**
 * CLX builder exception - thrown when a problem occured when building a CLX object.
 * <p>
 * It is a simple extension of {@link java.lang.Exception} class. It gives one additional 
 * attribute ({@link #getPath()}) that stores path to CLX element that caused the problem
 * (for instance <code>/rules/rule/forAll</code>). If this value is not specified at 
 * creation time, then it takes a default value - <code>null</code>.
 * <br>
 * For detailed description about constructors please look directly to 
 * superclass ({@link Exception}).
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public class ClxBuilderException extends Exception {

    /** Path to CLX element that caused the problem. It can be null */
    private String path = null;
    
    /**
     * Constructs a new exception with <code>null</code> as its detail message and path.
     * @see Exception#Exception()
     */
    public ClxBuilderException() {
        super();
    }

    /**
     * Constructs a new exception with the specified detail message and message.
     * @param  message the detail message; can be null
     * @param  cause the cause; can be null
     * @see Exception#Exception(java.lang.String, java.lang.Throwable)
     */
    public ClxBuilderException(String message, Throwable cause) {
        super(message, cause);
    }

    /**
     * Constructs a new exception with the specified detail message.  T
     * @param message the detail message; can be null
     * @see Exception#Exception(java.lang.String)
     */
    public ClxBuilderException(String message) {
        super(message);
    }

    /**
     * Constructs a new exception with the specified cause and a detail
     * message of <tt>(cause==null ? null : cause.toString())</tt>.
     * @param  cause the cause; can be null
     * @see Exception#Exception(java.lang.Throwable)
     */
    public ClxBuilderException(Throwable cause) {
        super(cause);
    }
    
    /**
     * Constructs a new exception with the specified detail message and path.
     * @param message the detail message; can be null
     * @param path the CLX path; can be null
     * @see Exception#Exception(java.lang.String)
     */
    public ClxBuilderException(String message, String path) {
        super(message);
        this.path = path;
    }
    
    /**
     * Constructs a new exception with the specified detail message, path and cause.  
     * @param message the detail message; can be null
     * @param path the CLX path; can be null
     * @param cause the cause; can be null
     * @see Exception#Exception(java.lang.String, java.lang.Throwable)
     */
    public ClxBuilderException(String message, String path, Throwable cause) {
        super(message, cause);
        this.path = path;
    }
    
    /**
     * Returns path to CLX element that caused the problem.
     * @return path to CLX element, for instance <code>/rules/rule/forAll</code>; can be null
     */
    public String getPath() {
        return this.path;
    }
    
}
