package org.xdv.clx.exec;

/**
 * Execution point to resume CLX rule evaluation to find all errors.
 * <p>
 * It is used to perform <i>deep validation</i>.
 * Algorithm is simple:
 * <ol>
 *  <li>Execute rule's formula.
 *  <li>If it returns false (formula not valid) generate validation error and resume
 *      evaluation of the same formula to find more points (XML nodes) where this rule is violated.
 *  <li>CLX formula may store there any information it needs to resume evaluation after.
 * </ol>
 * </p>
 * 
 * @author Tomasz Kokoszka
 */
public class ClxExecutionPoint {

    /** Execution point for first sub-formula */
    protected ClxExecutionPoint next1;
    
    /** Execution point for secound sub-formula */
    protected ClxExecutionPoint next2;
    
    /** Is this execution point finished (false by default) ? 
     * If it is finished it doesn't need to be resumed - it will not return any other errors */
    protected boolean finished = false;
    
    /** Additional data in this execution point - it can store any data depending on formula */
    protected Object data = null;        
              
    /**
     * Returns additional data.
     * @return additional data; can be null
     */
    public Object getData() {
        return data;
    }

    /**
     * Sets additional data.
     * @param data additional data; can be null
     */
    public void setData(Object data) {
        this.data = data;
    }

    /**
     * Returns execution point for first sub-formula, if execution point doesn't exist it creates
     * a new one.
     * @return execution point for first sub-formula; !=null
     */
    public ClxExecutionPoint getNext1() {
        if (next1 == null) {
            next1 = new ClxExecutionPoint();
        }
        return next1;
    }

    /**
     * Sets new execution point for first sub-formula. 
     * @param next1 execution point for first sub-formula; can be null
     */
    public void setNext1(ClxExecutionPoint next1) {
        this.next1 = next1;
    }
    
    /**
     * Returns execution point for secound sub-formula, if execution point doesn't exist it creates
     * a new one.
     * @return execution point for secound sub-formula; !=null
     */
    public ClxExecutionPoint getNext2() {
        if (next2 == null) {
            next2 = new ClxExecutionPoint();
        }
        return next2;
    }

    /**
     * Sets new execution point for secound sub-formula. 
     * @param next2 execution point for secound sub-formula; can be null
     */
    public void setNext2(ClxExecutionPoint next2) {
        this.next2 = next2;
    }

    /**
     * Is this execution point finished? 
     * If it is finished it doesn't need to be resumed - it will not return any other errors
     * @return true if this execution point is finished and should not be resumed
     */
    public boolean isFinished() {
        return this.finished;
    }
    
    /**
     * Finish this execution point.
     * It is a message to the parent: do not resume this execution point because it has nothing more
     * to execute.
     */
    public void finish() {
        this.finished = true;
    }
    
    /**
     * Sets finished flag to given value.
     * @param finished true means that this execution point is finished
     */
    public void setFinished(boolean finished) {
        this.finished = finished;
    }
    
    /**
     * Sets finished flag according to value of this flags in two sub-formulas.
     * Sets finished to true if 
     * <code>(next1==null || next1.isFinished()) && (next2==null || next2.isFinished())</code>
     *
     */
    public void setFinished() {
        boolean b = true;
        if (next1 != null) {
            b &= next1.isFinished();
        }
        if (next2 != null) {
            b &= next2.isFinished();
        }
        this.finished = b;
    }
    
}
