package org.xdv.clx.expr;

import java.util.Iterator;
import java.util.ListIterator;

import org.xdv.clx.base.ClxConstants;
import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.exec.ClxExecutionPoint;
import org.xdv.common.XdvValidationException;
import org.xdv.xpath.XPathExpression;
import org.xdv.xpath.XPathValue;
import org.xdv.xpath.XPathValueNodeSet;

/**
 * EXISTS - CLX element implementation.
 * <br>
 * It is a quantifier. To evaluate its value:
 * <ol>
 *  <li>Execute XPath expression and get result as a node-set.
 *  <li>For every XML node from node-set:
 *      <ol>
 *          <li>Bind it to a variable,
 *          <li>Execute sub-formula,
 *          <li>If sub-formula evaluates to <code>true</code> return <code>true</code> 
 *              immediately,
 *      </ol>
 *  <li>Return <code>false</code>
 * </ol>   
 * 
 * @see <a href="http://www.clixml.org/clix/1.0/#exists">CLiXML - clx:exists</a>
 * @see org.xdv.clx.base.ClxConstants#TAG_EXISTS
 * @author Tomasz Kokoszka
 */  
public class ClxExists extends ClxFormulaQuantifier {

    /**
     * Creates instance.
     * @param var variable's name
     * @param in XPath expression that is evaluated to a node-set
     * @param formula sub-formula
     */
    public ClxExists(String var, XPathExpression in, ClxFormula formula) {
        super(var, in, formula);
    }
    
    public String getName() {
        return ClxConstants.TAG_EXISTS;
    }

    public boolean executeFormula(ClxExecutionContext cntx) 
    throws XdvValidationException {
        XPathValueNodeSet nodes = getInNodeSet(cntx);
        
        cntx.declareVariable(getVar());
        
        boolean valid = false;
        for(Iterator i = nodes.getNodesAsValues().iterator(); i.hasNext() && !valid; ) {
            final XPathValue node = (XPathValue) i.next();            
            cntx.setVariable(getVar(), node);
            valid = getFormula().executeFormula(cntx);
        }
        
        cntx.undeclareVariable(getVar());
               
        return valid;
    }
    
    public boolean executeFormula(ClxExecutionContext cntx, ClxExecutionPoint frame) 
    throws XdvValidationException {
        ListIterator iter = getExecutionState(cntx, frame);
        ClxExecutionPoint nextFrame = frame.getNext1();
        
        cntx.declareVariable(getVar());
                    
        boolean valid = false;
        boolean createNewFrame = false;
        while (iter.hasNext()) {
            final XPathValue node = (XPathValue) iter.next();            
            cntx.setVariable(getVar(), node);
            
            if (createNewFrame) {
                nextFrame = new ClxExecutionPoint();
            } else {
                createNewFrame = true;
            }
            
            valid = getFormula().executeFormula(cntx, nextFrame);
            if (valid) {
                if (! nextFrame.isFinished()) {
                    iter.previous();
                } else {
                    nextFrame = new ClxExecutionPoint();
                }
                break;
            }
        }
        
        frame.setNext1(nextFrame);
        if (! iter.hasNext()) {
            frame.finish();            
        }
        
        cntx.undeclareVariable(getVar());
               
        return valid;
    }        
    
}
 