package org.xdv.clx.expr;

import java.util.List;

import org.xdv.clx.base.ClxValidationError;
import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.exec.ClxExecutionPoint;
import org.xdv.clx.extension.ClxExtErrorReport;
import org.xdv.common.XdvErrorLevelSet;
import org.xdv.common.XdvValidationException;

/**
 * RULE - CLX element implementation.
 * <br>
 * It is a definition of a single validation rule.
 * 
 * @see <a href="http://www.clixml.org/clix/1.0/#rule.schema">CLiXML - clix:rule</a>
 * @see org.xdv.clx.base.ClxConstants#TAG_RULE
 * @author Tomasz Kokoszka
 */
public class ClxRule {
    
    /** Rule ID */
    private final String id;
    
    /** Is rule disabled?
     * Disabled rules are not taken in consideration during validation, in other words: they
     * are always valid. */
    private final boolean disabled;
    
    /** Rule header - null means no header */
    private final ClxHeader header;
    
    /** Error reporter; !=null */
    private final ClxExtErrorReport report;
    
    /** Formula; !=null */
    private final ClxFormula formula;

    /**
     * Create rule.
     * @param id rule ID
     * @param disabled is rule disabled? Disabled rules are not taken in consideration during 
     *        validation, in other words: they are always valid. 
     * @param header header, can be null
     * @param report error reporter
     * @param formula formula that describes this rule
     */
    public ClxRule(String id, boolean disabled, 
            ClxHeader header, 
            ClxExtErrorReport report, 
            ClxFormula formula) {
        this.id = id;
        this.disabled = disabled;
        this.header = header;
        this.report = report;
        this.formula = formula;
    }       

    /**
     * Is rule disabled?
     * Disabled rules are not taken in consideration during validation, in other words: they
     * are always valid.
     * @return value <code>true</code> means that this rule is disabled
     */
    public boolean isDisabled() {
        return disabled;
    }

    /**
     * Return first order logic formula.
     * @return first order logic formula; !=null
     */
    public ClxFormula getFormula() {
        return formula;
    }

    /** 
     * Returns header.
     * @return header, null means there is no header
     */
    public ClxHeader getHeader() {
        return header;
    }

    /**
     * Returns rule ID.
     * @return rule ID; !=null
     */
    public String getId() {
        return id;
    }

    /**
     * Returns error reporter.
     * @return error reporter; !=null
     */
    public ClxExtErrorReport getReport() {
        return report;
    }
    
    /**
     * Executes rule - general validation till first error.
     * @param cntx execution context
     * @return validation result (<code>true</code> means valid)
     * @throws XdvValidationException error while executing rule
     * @see ClxRules#execute(ClxExecutionContext, XdvErrorLevelSet)
     */
    public boolean execute(ClxExecutionContext cntx) 
    throws XdvValidationException {
        if (disabled) {
            return true;
        }        
        
        return formula.executeFormula(cntx);            
    }
    
    /**
     * Executes rule - simple validation till first error.
     * @param cntx execution context
     * @return first error reported during validation or <code>null</code> if no errors have been
     *         reported
     * @throws XdvValidationException error while executing rule
     * @see ClxRules#executeSimple(ClxExecutionContext, XdvErrorLevelSet)
     */
    public ClxValidationError executeSimple(ClxExecutionContext cntx) 
    throws XdvValidationException {
        if (disabled) {
            return null;
        }        
        
        boolean valid = formula.executeFormula(cntx);
        if (! valid) {
            return report.generateError(cntx, this);
        }        
        
        return null;
    }
        
    /**
     * Executes rule - deep validation.
     * It doesn't stop on first error, but after formula fails it resumes its evaluation to find
     * more places where same formula might be violated.
     * @param cntx execution context
     * @param errors list of errors; modified in place. Method will add all reported 
     *        validation errors at the end of this list
     * @throws XdvValidationException error while executing rule
     * @see ClxRules#executeDeep(ClxExecutionContext, List, XdvErrorLevelSet)
     */
    public void executeDeep(ClxExecutionContext cntx, List errors) 
    throws XdvValidationException {
        if (disabled) {
            return;
        }        
        
        ClxExecutionPoint frame = new ClxExecutionPoint();
        boolean valid = false;
        
        while (!valid && !frame.isFinished()) {
            valid = formula.executeFormula(cntx, frame);
            if (! valid) {
                errors.add(report.generateError(cntx, this));
            }
        }
    }
    
    public String toString() {
        return (this.disabled?"-":"") + this.id;
    }

}
