package org.xdv.clx.expr;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.xdv.clx.base.ClxValidationError;
import org.xdv.clx.base.ClxValidator;
import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.clx.extension.ClxExtMacroCall;
import org.xdv.clx.operator.ClxOperatorCall;
import org.xdv.common.XdvErrorLevelSet;
import org.xdv.common.XdvValidationException;
import org.xdv.xpath.XPathException;
import org.xdv.xpath.XPathValue;

/**
 * RULES - CLX element implementation.
 * <br>
 * It is a set of rule definitions.
 * 
 * @see <a href="http://www.clixml.org/clix/1.0/#rules.schema">CLiXML - clix:rules</a>
 * @see org.xdv.clx.base.ClxConstants#TAG_RULES
 * @author Tomasz Kokoszka
 */
public class ClxRules {

    /** List of external macros - list of {@link String}, where every entry is an URL to 
     * XML file containing macro definitions; !=null */
    private final List macrosURL;
    
    /** List of rules variables - list of {@link ClxVariable}; !=null */
    private final List variables;
    
    /** List of key-maps - list of {@link ClxKey}; !=null */
    private final List keys;
    
    /** Map of all rules in this rule-set: rule ID-&gt;rule definition 
     * ({@link String}-&gt;{@link ClxRule}) */
    private final Map rules;
    
    /**
     * All macro's calls from all forumals inside this rule-set.
     * Collection of {@link ClxExtMacroCall}; !=null.
     */
    private Collection macroCalls = new ArrayList(0);
    
    /**
     * All operator's calls from all formulas inside this rule-set.
     * Collection of {@link ClxOperatorCall}; !=null.
     */
    private Collection operatorCalls = new ArrayList(0);
       
    /**
     * Create an empty rule-set.
     * @param macrosURL external macros - list of {@link String}, where every entry is an URL to 
     *        XML file containing macro definitions
     * @param variables variables - list of {@link ClxVariable}
     * @param keys key-map definitions - list of {@link ClxKey}
     */
    public ClxRules(List macrosURL, List variables, List keys) {
        this.macrosURL = macrosURL;
        this.variables = variables;
        this.keys = keys;
        this.rules = new LinkedHashMap();
    }                
        
    /**
     * Returns list of key-maps - list of {@link ClxKey}.
     * @return list of key-maps - list of {@link ClxKey}; !=null
     */
    public List getKeys() {
        return keys;
    }

    /**
     * Returns list of external macros.
     * @return list of external macros - list of {@link String}, where every entry is an URL to 
     *         XML file containing macro definitions; !=null
     */
    public List getMacrosURL() {
        return macrosURL;
    }

    /**
     * Returns list of variables.
     * @return list of variables - list of {@link ClxVariable}; !=null
     */
    public List getVariables() {
        return variables;
    }
    
    /**
     * Returns rule with given ID from set.
     * @param ruleId rule ID
     * @return rule or <code>null</code> if there is no rule with given ID in set
     */
    public ClxRule getRule(String ruleId) {
        return (ClxRule) rules.get(ruleId);
    }
    
    /**
     * Returns all rules from set.
     * @return collection of {@link ClxRule}; !=null
     */
    public Collection getRules() {
        return rules.values();
    }
    
    /**
     * Returns collections of rule's ID from set.
     * @return set of rule's ID (set of {@link String})
     */
    public Set getRuleIds() {
        return rules.keySet();
    }
    
    /** 
     * Adds rule to this set.
     * @param rule rule to add
     * @return rule that was before in set under same rule's ID, or <code>null</code> if set
     *         didn't have a rule with same ID before
     */
    public ClxRule addRule(ClxRule rule) {
        return (ClxRule) rules.put(rule.getId(), rule);
    }
    
    /**
     * Adds all rules from collection to this set.
     * @param ruleCol collections of rules to add (collection of {@link ClxRule})
     * @return rule that was before in set under same rule's ID, or <code>null</code> if set
     *         didn't have a rule with same ID before
     */
    public ClxRule addRuleAll(Collection ruleCol) {
        ClxRule duplicate = null;
        for(Iterator i = ruleCol.iterator(); i.hasNext(); ) {
            ClxRule old = addRule((ClxRule) i.next());
            if (old != null) {
                duplicate = old;
            }
        }
        return duplicate;
    }

    /**
     * Returns all macro's calls from all forumals inside this rule-set.
     * @return collection of {@link ClxExtMacroCall}; !=null
     */
    public Collection getMacroCalls() {
        return macroCalls;
    }

    /**
     * Sets internal colection of macro's calls.
     * @param macroCalls collection of {@link ClxExtMacroCall}
     */
    public void setMacroCalls(Collection macroCalls) {
        this.macroCalls = macroCalls;
    }

    /**
     * Returns all operator's calls from all forumals inside this rule-set.
     * @return collection of {@link ClxOperatorCall}; !=null
     */
    public Collection getOperatorCalls() {
        return operatorCalls;
    }

    /**
     * Sets internal colection of operator's calls.
     * @param operatorCalls collection of {@link ClxOperatorCall}
     */
    public void setOperatorCalls(Collection operatorCalls) {
        this.operatorCalls = operatorCalls;
    }

    /**
     * Evaluates, declares and sets variables ({@link #variables}) in context.
     * @param cntx execution context; modified in place
     * @throws XdvValidationException error while evaluating variable
     */
    private void setVariables(ClxExecutionContext cntx) 
    throws XdvValidationException {
        for(Iterator i = variables.iterator(); i.hasNext(); ) {
            final ClxVariable var = (ClxVariable) i.next();
            XPathValue value;
            try {
                value = cntx.executeXPath(var.getXpath());
            } catch (XPathException e) {
                throw new XdvValidationException("Unable to evaluate variable " + var.getId(), e);
            }
            
            cntx.declareVariable(var.getId());
            cntx.setVariable(var.getId(), value);
        }
    }
    
    /**
     * Undeclares variables ({@link #variables}) in context.
     * @param cntx execution context; modified in place
     * @throws XdvValidationException error while undeclaring variable
     */
    private void removeVariables(ClxExecutionContext cntx) 
    throws XdvValidationException {
        for(Iterator i = variables.iterator(); i.hasNext(); ) {
            final ClxVariable var = (ClxVariable) i.next();
            cntx.undeclareVariable(var.getId());
        }
    }

    /**
     * Executes rules from this set till first error - general validation agains rules from
     * this set.
     * @param cntx execution context
     * @param errorSet error-set - method will execute only rules that can report validation-error
     *        from this error-set
     * @return validation result (<code>true</code> means valid)
     * @throws XdvValidationException error while executing rule
     * @see ClxValidator#validate(Object, XdvErrorLevelSet)
     */
    public boolean execute(ClxExecutionContext cntx, XdvErrorLevelSet errorSet) 
    throws XdvValidationException {
        boolean valid = true;
        
        setVariables(cntx);        
        for(Iterator i = rules.values().iterator(); i.hasNext() && valid; ) {
            final ClxRule rule = (ClxRule) i.next();
            if (errorSet.contains(rule.getReport().getErrorLevel())) {
                valid = rule.execute(cntx);
            }
            cntx.removeUndeclared();
        }        
        removeVariables(cntx);
        
        return valid;
    }        
    
    /**
     * Executes rules from this set till first error - simple validation agains rules from
     * this set.
     * @param cntx execution context
     * @param errorSet error-set - method will execute only rules that can report validation-error
     *        from this error-set
     * @return first error reported during validation or <code>null</code> if no errors have been
     *         reported
     * @throws XdvValidationException error while executing rule
     * @see ClxValidator#validateSimple(Object, XdvErrorLevelSet)
     */
    public ClxValidationError executeSimple(ClxExecutionContext cntx, XdvErrorLevelSet errorSet) 
    throws XdvValidationException {
        ClxValidationError error = null;
        
        setVariables(cntx);        
        for(Iterator i = rules.values().iterator(); i.hasNext() && (error == null); ) {
            final ClxRule rule = (ClxRule) i.next();
            if (errorSet.contains(rule.getReport().getErrorLevel())) {
                error = rule.executeSimple(cntx);
            }
            cntx.removeUndeclared();
        }        
        removeVariables(cntx);
        
        return error;
    }
    
    /**
     * Executes rules from this set - deep validation.
     * It doesn't stop on first error, but after rule fails it resumes its evaluation to find
     * more places where same rule might be violated. 
     * @param cntx execution context
     * @param errors list of errors; modified in place. Method will add all reported 
     *        validation errors at the end of this list
     * @param errorSet error-set - method will execute only rules that can report validation-error
     *        from this error-set
     * @throws XdvValidationException error while executing rule
     * @see ClxValidator#validateDeep(Object, XdvErrorLevelSet)
     */
    public void executeDeep(ClxExecutionContext cntx, List errors, XdvErrorLevelSet errorSet) 
    throws XdvValidationException {
        setVariables(cntx);        
        for(Iterator i = rules.values().iterator(); i.hasNext(); ) {
            final ClxRule rule = (ClxRule) i.next();
            if (errorSet.contains(rule.getReport().getErrorLevel())) {
                rule.executeDeep(cntx, errors);
            }
            cntx.removeUndeclared();
        }        
        removeVariables(cntx);        
    }
    
    /**
     * Prints collection into buffer (elements are separated using space character).
     * @param list collection to print
     * @param out destination buffer; modified in place
     */
    private void printList(Collection list, StringBuffer out) {
        for(Iterator i = list.iterator(); i.hasNext(); ) {
            out.append(i.next());
            if (i.hasNext()) {
                out.append(" ");
            }
        }   
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("{{");
        printList(macrosURL, sb);
        sb.append("} {");
        printList(variables, sb);
        sb.append("} {");
        printList(keys, sb);
        sb.append("} {");
        printList(rules.values(), sb);
        sb.append("}}");
        return sb.toString();
    }
    
}
