package org.xdv.clx.operator;

import java.util.Iterator;
import java.util.List;

import org.xdv.clx.builder.ClxBuilderException;
import org.xdv.clx.exec.ClxExecutionContext;
import org.xdv.common.XdvValidationException;
import org.xdv.xpath.XPathEngine;
import org.xdv.xpath.XPathException;
import org.xdv.xpath.XPathExpression;

/**
 * Plugin operator - xpath condition (factory).
 * <p>
 * <b>Behavior:</b><br>
 * Operator evaluates to <code>true</code> if and only if given XPath expression 
 * (operator parameter) evaluates to value that after converting to boolean gives <code>true</code>.
 * <br><b>Parameters:</b><br>
 * <ul>
 *  <li>xpath - (required) XPath expression
 * </ul>
 * </p>
 * 
 * <p>
 * <b>Example:</b>
 * <pre>
 *  &lt;operator name="xpathCondition"&gt;
 *      &lt;!-- variable must be defined somewhere before to make it work --&gt;
 *      &lt;param name="xpath" value="/data/field/@name='signal' and $a='global-data'"/&gt;
 *  &lt;/operator&gt;
 * </pre>
 * </p>
 * 
 * <p>
 * It is a simple implementation of plugin operator - factory and definition. 
 * It completely hides operator definition, only factory is visible. 
 * You should take a look at it before creating your first operator.
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public class ClxXPathCondition implements ClxOperatorDefFactory {        
    
    /** Name of operator parameter that holds XPath expression */
    private static final String PARAM_XPATH = "xpath";
    
    /** Operator name */
    static final String MACRO_NAME = "xpathCondition";
    
    /** Factory instance */
    private static final ClxOperatorDefFactory FACTORY = new ClxXPathCondition();
    
    /**
     * Returns factory to instantiate this plugin operator.
     * @return factory instance; !=null. As factory doesn't have a state it always returns exacly
     *         the same object
     */
    public static ClxOperatorDefFactory getFactory() {
        return FACTORY;
    }
    
    /**
     * Creates a factory instance ({@link ClxOperatorDefFactory}).
     * To get factory instance to register it in validator factory use 
     * {@link #getFactory()}.
     */
    private ClxXPathCondition() {
    }
    
    public String getName() {
        return MACRO_NAME;
    }
    
    public ClxOperatorDef createCall(List params, XPathEngine engine) 
    throws ClxBuilderException, XPathException {
        String xpathVal = null;
        for(Iterator i = params.iterator(); i.hasNext(); ) {
            final ClxOperatorParam param = (ClxOperatorParam) i.next();
            if (PARAM_XPATH.equals(param.getName())) {
                if (xpathVal == null) {
                    xpathVal = param.getValue();
                } else {
                    throw new ClxBuilderException("Parameter " + param.getName() + 
                            " defined more than once for operator " + getName());
                }
            } else {
                throw new ClxBuilderException("Unknown parameter - " + param.getName() + 
                        " - not recognized by operator " + getName());
            }
        }
        
        if (xpathVal == null) {
            throw new ClxBuilderException("Parameter " + PARAM_XPATH + " is missing for " +
                    "operator " + getName());
        }
        
        return new ClxXPathConditionDef(engine.createExpression(xpathVal));
    }    
        
}

/**
 * Plugin operator - xpath condition (definition).
 * Instances should be created using factory - {@link ClxXPathCondition}.
 *
 * @author Tomasz Kokoszka
 */
class ClxXPathConditionDef extends ClxOperatorDef {        
                
    /** Operator parameter - XPath expression */
    private final XPathExpression xpath;
        
    /**
     * Creates an operator instance.
     * Operator should be instantiated only using factory. 
     * @param xpath XPath expression
     */
    ClxXPathConditionDef(XPathExpression xpath) {
        //for implementation
        this.xpath = xpath;
    }
        
    public boolean executeFormula(ClxExecutionContext cntx) 
    throws XdvValidationException {
        return executeXPath(xpath, cntx).asBoolean();
    }
    
    public String toString() {
        return ClxXPathCondition.MACRO_NAME + "(" + xpath + ")";
    }
    
}

