package org.xdv.common;

import java.util.Collection;

/**
 * General validation API - interface to validate XML documents and collect results.
 * <p>
 * This interface defines general validation API.
 * It introduces several ways to validate document:
 * <ul class="NoTopMargin">
 *  <li><b>General validation</b> - gives simple answer: 
 *      <i>valid</i>/<i>not valid</i> ({@link #validate(Object)}),
 *  <li><b>Simple validation</b> - gives one reported error ({@link #validateSimple(Object)}).
 *      <br> 
 *      Usually it will be the first reported error, but it depends on implementation,
 *  <li><b>Deep validation</b> - gives list of all reported errors ({@link #validateDeep(Object)}).
 * </ul>
 * You should be aware that usually <i>deep validation</i> is slower than <i>simple validation</i>
 * which is slower than <i>general validation</i>. 
 * So, don't use <i>deep validation</i> just to check if document is valid or not 
 * (it would be just a waste of CPU time).
 * <br> 
 * There is a posibility to report errors only from given error-level: 
 * <ul class="NoTopMargin">
 *  <li>{@link #validate(Object, XdvErrorLevelSet)}
 *  <li>{@link #validateSimple(Object, XdvErrorLevelSet)}
 *  <li>{@link #validateDeep(Object, XdvErrorLevelSet)}
 * </ul>
 * </p>
 * <p>
 * Interface doesn't specify type of source document.
 * It could be a W3C DOM document, JDOM document or anything else - it depends on implementation.
 * If you pass object that is not valid for underlying implementation 
 * then an {@link XdvValidationError} will be thrown.  
 * </p>
 * 
 * @author Tomasz Kokoszka
 */
public interface XdvValidator {

    /**
     * Validates XML document - gives a simple answer: valid/not valid.
     * @param xmlDocument XML document to validate
     * @param errorSet set of error-levels, only errors with error-level from this set are
     *        taken in consideration during validation 
     * @return Value <code>true<code> if and only if there were no errors
     * @throws XdvValidationException unable to perform validation
     */
    public boolean validate(Object xmlDocument, XdvErrorLevelSet errorSet) 
    throws XdvValidationException;
    
    /**
     * Validates XML document - gives one reported error or <code>null</code> if there are no 
     * errors.
     * @param xmlDocument XML document to validate
     * @param errorSet set of error-levels, only errors with error-level from this set are
     *        taken in consideration during validation
     * @return Any error with error-level from <code>errorSet</code> reported during validation
     *         or <code>null</code> if there were no such errors.
     *         Usually it will be the first reported error but it depends on implementation.
     * @throws XdvValidationException unable to perform validation
     */
    public XdvValidationError validateSimple(Object xmlDocument, XdvErrorLevelSet errorSet) 
    throws XdvValidationException;

    /**
     * Validates XML document - gives list of all reported errors.
     * @param xmlDocument XML document to validate
     * @param errorSet set of error levels, only errors with error-level from this set are
     *        taken in consideration during validation
     * @return Collection of {@link XdvValidationError}; !=null. 
     *         All errors with error-level from <code>errorSet</code> reported during validation
     *         or an empty list if there were no such errors.         
     * @throws XdvValidationException unable to perform validation
     */
    public Collection validateDeep(Object xmlDocument, XdvErrorLevelSet errorSet) 
    throws XdvValidationException;

    /**
     * Validates XML document - gives a simple answer: valid/not valid.
     * @param xmlDocument XML document to validate
     * @return Value <code>true<code> if and only if there were no errors
     * @throws XdvValidationException unable to perform validation
     */
    public boolean validate(Object xmlDocument) 
    throws XdvValidationException;

    /**
     * Validates XML document - gives one reported error or <code>null</code> if there are no 
     * errors.
     * @param xmlDocument XML document to validate
     * @return Any error reported during validation or <code>null</code> if there were no errors.
     *         Usually it will be the first reported error but it depends on implementation.
     * @throws XdvValidationException unable to perform validation
     */
    public XdvValidationError validateSimple(Object xmlDocument) 
    throws XdvValidationException;

    /**
     * Validates XML document - gives list of all reported errors.
     * @param xmlDocument XML document to validate
     * @return Collection of {@link XdvValidationError}; !=null. 
     *         All errors reported during validation or an empty list if there were no errors.
     * @throws XdvValidationException unable to perform validation
     */
    public Collection validateDeep(Object xmlDocument) 
    throws XdvValidationException;
    
    /**
     * Returns type of supported XML document.
     * @return Type/class of supported XML document sources (for information purposes); !=null.
     *         For instance - {@link org.w3c.dom.Document}.
     */
    public Class getDocumentClass();
    
}
