package org.xdv.xpath.common;

import org.xdv.xpath.XPathValue;

/**
 * XPath boolean value - wraps {@link java.lang.Boolean}.
 * It is a simple wrapper for {@link Boolean}. 
 * Can be used by any XPath implementation that maps {@link Boolean} to XPath 1.0 boolean type. 
 * <p>
 * Instance should never be created directly but rather using suitable XPath engine.
 * Instances are immutable - it means that its value cannot be changed.
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public class BaseXPathValueBoolean implements XPathValue {
    
    /** Wrapped value - immutable. !=null */
    private final Boolean value;
    
    /**
     * Creates wrapper for given {@link Boolean} object.
     * @param value object to wrap
     */
    public BaseXPathValueBoolean(Boolean value) {
        assert value != null;
        this.value = value;
    }
    
    /**
     * Returns wrapped object.
     * @return Wrapped object, !=null
     */
    public Boolean getValue() {
        return value;
    }

    /**
     * Returns wrapped object - instance of {@link Boolean}.
     * @return Wrapped object - {@link Boolean}, !=null
     */
    public Object getNative() {
        return value; 
    }

    /**
     * Returns value type.
     * @return {@link XPathValue#TYPE_BOOLEAN}
     */
    public int getType() {
        return XPathValue.TYPE_BOOLEAN;
    }
    
    public String asString() {
        return value.toString();
    }

    public boolean asBoolean() {
        return value.booleanValue();
    }

    public double asNumber() {
        return value.booleanValue()?1:0;
    }
    
    public String toString() {
        return value.toString();
    }
    
}
