package org.xdv.xpath.common;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xdv.xpath.XPathValue;
import org.xdv.xpath.XPathValueNodeSet;

/**
 * XPath DOM single node - wraps {@link org.w3c.dom.Node}.
 * It is a wrapper for {@link Node}. 
 * Can be used by any XPath implementation that maps {@link Node} to an single size node-set. 
 * <p>
 * Instance should never be created directly but rather using suitable XPath engine.
 * Instances are immutable - it means that its value cannot be changed.
 * </p>
 * 
 * @author Tomasz Kokoszka
 */
public class DOMXPathValueNode implements XPathValueNodeSet {
    
    /**
     * Returns node text value.
     * Value depends on node type:
     * <ol class="NoTopMargin">
     * <li>element - concatenated value of all text nodes children of the element
     * <li>attribute - value of attribute
     * <li>text node - content of text node
     * <li>comment - content of the comment, excluding the <code>&lt;!--</code> and
     *     <code>--&gt;</code> escape characters
     * <li>processing instruction - content of the processing instruction following the target,
     *     but excluding the terminating <code>?&gt;</code>
     * </ol>
     *      
     * @param node XML node
     * @return Node text value; !=null
     */ 
    public static String toString(Node node) {
        switch(node.getNodeType()) {        
        case Node.ATTRIBUTE_NODE:
        case Node.PROCESSING_INSTRUCTION_NODE:            
            return node.getNodeValue();
        case Node.TEXT_NODE:
        case Node.CDATA_SECTION_NODE:
        case Node.COMMENT_NODE:
            return node.getNodeValue().trim();
        case Node.ELEMENT_NODE:
            final StringBuffer sb = new StringBuffer();
            final NodeList nodes = node.getChildNodes();
            final int size = nodes.getLength();
            for(int i = 0; i < size; i++) {
                final Node n = nodes.item(i);
                switch (n.getNodeType()) {
                case Node.TEXT_NODE:
                case Node.CDATA_SECTION_NODE:
                    sb.append(n.getNodeValue().trim());
                }
            }
            return sb.toString();
        }
        return node.getNodeValue();
    }
    
    /** Wrapped node - immutable. !=null */
    private final Node node;       
    
    /**
     * Creates wrapper for given {@link Node} object.
     * @param node XML node to wrap
     */
    public DOMXPathValueNode(Node node) {
        assert node != null;
        this.node = node;
    }    
    
    /**
     * Returns wrapped object.
     * @return Wrapped object, !=null
     */
    public Node getValue() {
        return this.node;
    }

    /**
     * Returns wrapped object - instance of {@link Node}.
     * @return Wrapped object - {@link Node}, !=null
     */
    public Object getNative() {
        return node;
    }

    /**
     * Returns wrapped document node - instance of {@link Node}.
     * @return XML node; !=null. Instance of {@link Node}  
     */
    public Object getFirstNode() {
        return this.node;
    }

    /**
     * Returns <code>true</code> if the specified XML node is the same as the only node 
     * in this node-set. 
     * @param nodeNative XML node whose presence in this node-set is to be tested
     * @return Value <code>true</code> if and only if the specified XML node is the same as 
     *         the only node in this node-set. 
     */
    public boolean containsNode(Object nodeNative) {
        return this.node.equals(nodeNative);
    }

    /**
     * Returns number of XML nodes in this node-set - always <b>1</b>.
     * @return Always <b>1</b> (one)
     */
    public int getLength() {
        return 1;
    }
    
    /**
     * Returns <code>false</code>.
     * @return Always <code>false</code> (not empty)
     */
    public boolean isEmpty() {
        return false;
    }

    /**
     * Returns list of XML nodes from this node-set - always one node.
     * @return List of {@link Node}, always contains 1 (one) element; !=null
     */
    public List getNodes() {
        final List l = new ArrayList(1);
        l.add(this.node);
        return l;
    }
        
    /**
     * Returns list of wrapped XML nodes from this node-set - always one object (<code>this</code>).
     * @return List of {@link DOMXPathValueNode}, 
     *         always contains 1 (one) element - <code>this</code>; !=null
     */
    public List getNodesAsValues() {
        final List l = new ArrayList(1);
        l.add(this);
        return l;
    }

    /**
     * Returns value type.
     * @return {@link XPathValue#TYPE_NODESET}
     */
    public int getType() {
        return XPathValue.TYPE_NODESET;
    }        
    
    public String asString() {
        return DOMXPathValueNode.toString(this.node);
    }

    public boolean asBoolean() {        
        return Boolean.valueOf(DOMXPathValueNode.toString(this.node)).booleanValue();
    }

    public double asNumber() throws NumberFormatException {
        try {
            return Double.parseDouble(DOMXPathValueNode.toString(this.node));
        } catch (NumberFormatException e) {
            return Double.NaN;
        }
    }
    
    public String toString() {
        return toString(this.node);
    }
    
}
