package org.xdv.xpath.jaxen;

import java.util.List;

import org.jaxen.JaxenException;
import org.jaxen.VariableContext;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xdv.xpath.XPathEngine;
import org.xdv.xpath.XPathException;
import org.xdv.xpath.XPathExecutionUnit;
import org.xdv.xpath.XPathExpression;
import org.xdv.xpath.XPathInvalidTypeError;
import org.xdv.xpath.XPathValue;
import org.xdv.xpath.XPathValueNodeSet;
import org.xdv.xpath.XPathVariableMap;

/**
 * Interface to execute Jaxen XPath expression(s).
 * <br>
 * Instance should be created using {@link JaxenXPathEngine#createExecutionUnit(Object)}.
 * Once created, it is always binded to one XML document, this binding cannot be changed.  
 * <p>
 * It works only with objects (expressions, values, etc.) created by Jaxen engine 
 * (see {@link org.xdv.xpath.jaxen.JaxenXPathEngine}).
 * </p>  
 *
 * @author Tomasz Kokoszka
 */
public class JaxenXPathExecutionUnit implements XPathExecutionUnit {
    
    /** XPath engine used to create this execution unit */
    private final JaxenXPathEngine engine;
        
    /** W3C DOM XML document - all XPath expressions will be executed <i>in</i> this document. */
    private final Document document;
    
    /** XPath variables map */
    private JaxenXPathVariableMap variables;
    
    /** Execution context - axis <i>self</i> for executed XPath expressions */
    private Node context;
       
    /**
     * Creates new execution unit binded to given document.
     * @param engine XPath engine suitable for this execution unit
     * @param doc W3C DOM XML document, all XPath expressions will be executed <i>in</i>
     *        this document
     */
    JaxenXPathExecutionUnit(JaxenXPathEngine engine, Document doc) {
        super();
        assert engine != null;
        assert doc != null;
        this.engine = engine;
        this.document = doc;
        this.context = doc.getDocumentElement();
        this.variables = engine.createJaxenVariableMap();
    }
    
    /**
     * Returns XPath implementation wrapper that was used to create this execution-unit.
     * @return XPath implementation wrapper; !=null. Instance of {@link JaxenXPathEngine}
     */
    public XPathEngine getEngine() {        
        return this.engine;
    }
        
    public void setContextRoot() {
        this.context = this.document.getDocumentElement();
    }

    /**
     * Sets <i>current</i> (axis <i>self</i>) element.
     * @param contextNative current node. It has to be an instance of {@link org.w3c.dom.Node}, 
     *        otherwise a {@link XPathInvalidTypeError} will be thrown.
     * @throws XPathInvalidTypeError object <code>contextNative</code> is not an instance of 
     *         {@link org.w3c.dom.Node}
     */
    public void setContext(Object contextNative) 
    throws XPathInvalidTypeError {
        if (! (contextNative instanceof Node)) {
            throw new XPathInvalidTypeError(contextNative, this.engine.getNodeClass());
        }

        this.context = (Node) contextNative;
    }

    /**
     * Returns variable map.
     * @return variable map; !=null. Instance of {@link JaxenXPathVariableMap}
     */
    public XPathVariableMap getVariables() {
        return this.variables;
    }    
    
    /**
     * Sets new variable map.
     * @param newVariables the variable map. It have to be an instance of 
     *        {@link JaxenXPathVariableMap}
     */    
    public void setVariables(XPathVariableMap newVariables) 
    throws XPathInvalidTypeError {
        if (! (newVariables instanceof JaxenXPathVariableMap)) {
            throw new XPathInvalidTypeError(newVariables, JaxenXPathVariableMap.class);
        }
        
        this.variables = (JaxenXPathVariableMap) newVariables;        
    }

    /**
     * Executes any Jaxen XPath expression.
     * <br> 
     * Type of result object depends of XPath type returned by XPath expression:
     * <ol class="NoTopMargin">
     * <li>String - returns instance of {@link org.xdv.xpath.common.BaseXPathValueString}
     * <li>Number - instance of {@link org.xdv.xpath.common.BaseXPathValueNumber}
     * <li>Boolean - instance of {@link org.xdv.xpath.common.BaseXPathValueBoolean}
     * <li>Node set - instance of {@link JaxenXPathValueNodeSet}
     * </ol> 
     * @param expression XPath expression. It have to be an instanceof {@link JaxenXPathExpression},
     *        otherwise a {@link XPathInvalidTypeError} is thrown
     * @return Result value, see conversion table above; !=null.
     * @throws XPathException error while executing xpath expression
     * @throws XPathInvalidTypeError expression is not an instance of {@link JaxenXPathExpression}
     */
    public XPathValue execute(XPathExpression expression)
    throws XPathException, XPathInvalidTypeError {
        if (! (expression instanceof JaxenXPathExpression)) {
            throw new XPathInvalidTypeError(expression, JaxenXPathExpression.class);
        }
        
        final JaxenXPathExpression expr = (JaxenXPathExpression) expression;        
        final VariableContext oldVars = expr.xpath.getVariableContext();
        expr.xpath.setVariableContext(this.variables);
        
        Object result;
        try {
            result = expr.xpath.evaluate(this.context);
        } catch(JaxenException e) {
            throw new XPathException(e.getMessage(), expr.asString(), e);
        } finally {
            expr.xpath.setVariableContext(oldVars);
        }
        
        return engine.createValue(result);
    }

    /**
     * Executes XPath expression that returns set of nodes.
     * @param expression XPath expression. It has to be an instanceof {@link JaxenXPathExpression},
     *        otherwise a {@link XPathInvalidTypeError} is thrown
     * @return Node set; !=null. Instance of {@link JaxenXPathValueNodeSet}
     * @throws XPathException error while executing xpath expression
     * @throws XPathInvalidTypeError expression is not an instance of {@link JaxenXPathExpression}
     */
    public XPathValueNodeSet executeSelect(XPathExpression expression)
    throws XPathException, XPathInvalidTypeError {
        if (! (expression instanceof JaxenXPathExpression)) {
            throw new XPathInvalidTypeError(expression, JaxenXPathExpression.class);
        }
        
        final JaxenXPathExpression expr = (JaxenXPathExpression) expression;
        final VariableContext oldVars = expr.xpath.getVariableContext();
        expr.xpath.setVariableContext(this.variables);
        
        Object result;
        try {
            result = expr.xpath.evaluate(this.context);
        } catch(JaxenException e) {
            throw new XPathException(e.getMessage(), expr.asString(), e);
        } finally {
            expr.xpath.setVariableContext(oldVars);
        }
        
        if (! (result instanceof List)) {
            throw new XPathException("This is not a node-set expression", expr.asString());
        }
        
        return new JaxenXPathValueNodeSet((List) result);
    }
    
}
