package org.xdv.xpath.jaxen;

import org.jaxen.UnresolvableException;
import org.jaxen.VariableContext;
import org.xdv.xpath.XPathValue;
import org.xdv.xpath.common.BaseXPathVariableMap;

/**
 * XPath variable map, mapping between variable name and variable value for Jaxen.
 * <p>
 * It is a simple extension to {@link org.xdv.xpath.common.BaseXPathVariableMap}. 
 * Additionally it implements methods required by Jaxen variable resolver - 
 * {@link org.jaxen.VariableContext}.
 * </p>
 *
 * @author Tomasz Kokoszka
 */
public class JaxenXPathVariableMap 
extends BaseXPathVariableMap implements VariableContext {

    /**
     * Creates an empty map.
     */
    JaxenXPathVariableMap() {
        super();
    }

    /**
     * Method from Jaxen variable context - used to resolve variable when executing
     * XPath expression using Jaxen.
     * <br>
     * If variable has not prefix (comes from <i>default</i> namespace) then parameter
     * <code>namespaceURI</code> is <code>null</code>.
     * <br>
     * Implementation:
     * <ol class="NoTopMargin">
     * <li>Call:
     *    <ul class="NoTopMargin">
     *    <li>{@link BaseXPathVariableMap#getVariable(String)} if <code>namespaceURI</code> is null
     *    <li>{@link BaseXPathVariableMap#getVariable(String, String)} otherwise
     *    </ul>
     * <li>If result is !=null return {@link XPathValue#getNative()}
     * <li>If result is null, throw {@link UnresolvableException}
     * </ol>
     * @param namespaceURI the namespace URI to which the variable belong to. 
     *        Value <code>null</code> means <i>default</i> namespace.
     * @param prefix the prefix that was used in the variable reference expression; this value is 
     *        ignored and has no effect
     * @param localName the local name of the variable-reference expression. If there is no prefix, 
     *        then this is the whole name of the variable. 
     * @return native variable value (see {@link XPathValue#getNative()})
     * @throws UnresolvableException variable is not in the map and cannot be resolved
     * @see org.jaxen.VariableContext#getVariableValue(java.lang.String, java.lang.String, java.lang.String)
     */
    public Object getVariableValue(String namespaceURI, String prefix, String localName) 
    throws UnresolvableException {
        if (namespaceURI == null) {
            final XPathValue v = getVariable(localName);
            if (v != null) {
                return v.getNative();
            }
        } else {
            final XPathValue v = getVariable(localName, namespaceURI);
            if (v != null) {
                return v.getNative();
            }
        }
        
        throw new UnresolvableException("Cannot resolve variable " + prefix + ":" + localName);
    }
    
}
